/*
 * Searchford Plugin
 * http://archtheme.com/searchford
 *
 * Advanced WordPress Search Plugin. Boost in-site search immediately and add live search previews.
 *
 */
;(function ( $, window, document, undefined ) {
        
        var pluginName = "searchford",
            settings = {
                min_chars: 3
            },
            grid_options = {
                srcNode: 'li',             // grid items (class, node)
                margin: '0',             // margin in pixel, default: 0px
                resizable: true,           // re-layout if window resize
                transition: 'none',
            };
            
        // The actual plugin constructor
        function Plugin( element, options, objD ) {
                var that = this;
                
                this.element = element;
                this.$element = $(element);
                this.$form = this.$element.closest('form');
                this.$input = this.$element.find('input[name="s"]');
                this._settings = settings;
                this._grid_options = $.extend( {}, grid_options, objD );
                this.not_found = false;
                this.req;
                
                var $sf = that.$element.find('input[name="sf"]'),
                    searchford_id = $sf.length > 0 ? $sf.val() : '';
                
                if( searchford_id.length > 0 ) {
                        if( that.$element.hasClass('searchford-live') ) {
                                that.$box = $('#searchford_box' + searchford_id);
                                that.$box.detach().appendTo('body');
                                that._box_width = that.$box.outerWidth();
                                that.init(that);
                        }
                } else {
                        $.post(
                                searchford_vars.ajaxurl, 
                                {
                                        action: 'searchford_box'
                                }, 
                                function(response){
                                        if( typeof(response) != 'undefined' ) {
                                                that._grid_options = $.extend( {}, that._grid_options, response.grid_options );
                                                that.$box = $(response.html);
                                                $('body').append(that.$box);
                                                that._box_width = that.$box.outerWidth();
                                                that.init(that);
                                        }
                                },
                                'json'
                        );
                }
        }
        
        Plugin.prototype = {
                init: function( that ) {
                        
                        var timer;

                        that.$form.attr('autocomplete', 'off');
                        that.$input.on( 'keyup', function(){
                                if( $(this).val().length >= that._settings.min_chars ) {

                                        if( that.not_found ) {
                                                that.load();
                                        }
                                        that.open(that);
                                        
                                        if (timer){
                                                clearTimeout(timer);
                                        }
                                        timer = setTimeout(function(){
                                                var data = that.$form.serialize();

                                                that.req = $.post(
                                                        searchford_vars.ajaxurl, 
                                                        data + '&action=searchford_ajax_query',
                                                        function(response){
                                                                if( response.results ) {
                                                                        that.success(response.results, response.style, that.$form.attr('action') + '?' + data);
                                                                } else if( response.suggestion ) {
                                                                        that.no_results(response.suggestion);
                                                                } else {
                                                                        that.no_results();
                                                                }
                                                        },
                                                        'json'
                                                );
                                        }, 500);
                                } else {
                                        that.close();
                                }
                        });
                },
                success: function( html, style, link ) {
                        this.not_found = false;
                        this.$box.find('.searchford-load').hide();
                        this.$box.find('.searchford-content').html(html).addClass('with-' + style);
                        this.$box.find('.searchford-foot').html('<a href="' + link + '">' + searchford_vars.search_results + '</a>');
                        this.$box.find('.searchford-masonry').gridify(this._grid_options);
                },
                load: function() {
                        this.$box.find('.searchford-content').html('');
                        this.$box.find('.searchford-load').show();
                        this.$box.find('.searchford-foot').html(searchford_vars.searching);
                },
                no_results: function(suggestion) {
                        
                        var error = typeof( suggestion ) == 'undefined' ? searchford_vars.no_posts : suggestion;
                        
                        this.not_found = true;
                        this.$box.find('.searchford-load').hide();
                        this.$box.find('.searchford-content').html('<div class="searchford-none"><em>' + error + '</em></div>');
                        this.$box.find('.searchford-foot').html(searchford_vars.no_results);
                },
                open: function(that) {
                        if( that.$box.is(':hidden') ) {
                                
                                var pos = that.$input.offset(),
                                    align = that.$box.data('searchford-align');
                                    input_width = that.$input.outerWidth(),
                                    window_width = $(window).width(),
                                    x = align == 'left' ? pos.left : window_width - pos.left - input_width,
                                    css = {},
                                    css['top'] = ( pos.top + that.$input.outerHeight() ) + 'px',
                                    css[align] = x + 'px';
                                    css['width'] = pos.left + that._box_width > window_width ? input_width + 'px' : '';

                                that.$box.css(css).show();
                                that._bindClick(that);
                        }
                },
                close: function() {
                        if( this.req != null ) this.req.abort();
                        
                        if( this.$box.is(':visible') ) {
                                this._unbindClick();
                                this.$box.hide();
                                this.$box.find('.searchford-content').html('');
                                this.$box.find('.searchford-load').show();
                                this.$box.find('.searchford-foot').html(searchford_vars.searching); 
                        }
                },
                _bindClick: function(that) {
                        $(document).on('click.searchford', function(e) {
                                if ( !that.$form.is(e.target) && that.$form.has(e.target).length === 0 &&
                                     !that.$box.is(e.target) && that.$box.has(e.target).length === 0) { // ... nor a descendant of the container 
                                        that.close();
                                }
                        });
                },
                _unbindClick: function() {
                        $(document).off('click.searchford');
                }
        };
        
        $.fn[pluginName] = function ( options ) {

                return this.each(function () {
                        if (!$.data(this, "plugin_" + pluginName)) {
                                var objD = $(this).data(pluginName);
                                
                                $.data(this, "plugin_" + pluginName,
                                new Plugin( this, options, objD ));
                        }
                });
        };
        
        jQuery(window).load(function(){
                $('form').has('input[name="s"]').searchford();
        });
        
})( jQuery, window, document );