<?php
/* ==========================================================================
   Register styles and scripts
   ========================================================================== */
if( !function_exists('searchford_register_scripts') ) {
        function searchford_register_scripts() {
                
                if( !is_admin() ) {
                        $blog_id = '';
                        if( is_multisite() ) {
                                $blog_id = get_current_blog_id();
                        }
                        
                        wp_enqueue_style('searchford_css', SEARCHFORD_URL . 'public/css/style.css');
                        
                        $css_file = SEARCHFORD_PATH . 'public/css/custom' . $blog_id . '.css';
                        if( file_exists( $css_file ) ) {
                                wp_enqueue_style('searchford_custom_css', SEARCHFORD_URL . 'public/css/custom' . $blog_id . '.css');
                        }
                        wp_enqueue_script('searchford_gridify', SEARCHFORD_URL . 'public/js/gridify.min.js', array('jquery'), '', true);
                        wp_enqueue_script('searchford_js', SEARCHFORD_URL . 'public/js/searchford.min.js', array('jquery'), '', true);
                        wp_localize_script('searchford_js', 'searchford_vars', array(
                                'ajaxurl' => admin_url('admin-ajax.php'),
                                'no_posts' => __('No posts found', SEARCHFORD_SLUG),
                                'no_results' => __('NO RESULTS FOUND', SEARCHFORD_SLUG),
                                'search_results' => __('SEARCH RESULTS', SEARCHFORD_SLUG),
                                'searching' => __('SEARCHING', SEARCHFORD_SLUG),
                                )
                        );
                }
        }
}

add_action('init', 'searchford_register_scripts');

/* ==========================================================================
   Filters
   ========================================================================== */
if ( !function_exists('searchford_link_action_on_plugin')) {
        function searchford_link_action_on_plugin( $links ) {
                $links[] = '<a href="'. get_admin_url(null, 'options-general.php?page=searchford-settings') .'">' . __('Settings', SEARCHFORD_SLUG) . '</a>';
                return $links;
        }
}

if ( !function_exists('searchford_add_var')) {
        function searchford_add_var( $public_query_vars ) {
                $public_query_vars[] = 'sf';
                return $public_query_vars;
        }
}

add_filter( 'plugin_action_links_searchford/searchford.php', 'searchford_link_action_on_plugin' );
add_filter('query_vars', 'searchford_add_var');
/* ==========================================================================
   Actions
   ========================================================================== */
if( !function_exists('searchford_orderby') ) {
        function searchford_orderby( $clause, $query ) {
                
                remove_filter(current_filter(), __FUNCTION__);
                
                global $wpdb;
                $post_table = $wpdb->prefix . 'posts';
                
                if( isset( $query->query_vars['searchford_order'] ) ) {
                        $order = $query->query_vars['searchford_order'];
                        $clause = "FIELD ($post_table.ID,$order) ASC";
                }

                return $clause;	
        }
}

if( !function_exists('searchford_where') ) {
        function searchford_where( $clause ) {
                remove_filter(current_filter(), __FUNCTION__);
                
                return '';
        }
}

if( !function_exists('searchford_highlight_term') ) {
        function searchford_highlight_term($content) {
                
                // get terms
                $term = get_query_var('s');
                $terms = preg_replace('!\s+!', '|', trim( $term ) ); //remove whitespace
                $terms = str_replace(array('+','(',')','>','<','~','*','"'), '', $terms); //remove operators

                if( $searchford_id = get_query_var('sf') ) {
                        $searchford_option = get_post_meta( $searchford_id, '_searchford_meta', true);
                        if( !$searchford_option ) {
                                $searchford_option = get_option( SEARCHFORD_OPTIONS );
                        }
                } else {
                        $searchford_option = get_option( SEARCHFORD_OPTIONS );
                }
                $markup_start = '';
                $markup_end = '';
                
                switch( $searchford_option['highlight_style'] ) {
                        case 'bold':
                                $markup_start = '<strong>';
                                $markup_end = '</strong>';
                                break;
                        case 'italics':
                                $markup_start = '<em>';
                                $markup_end = '</em>';
                                break;
                        case 'underline':
                                $markup_start = '<span style="text-decoration:underline;">';
                                $markup_end = '</span>';
                                break;
                        case 'color':
                                $style = '';
                                if( $searchford_option['highlight_text_color'] != '' ) {
                                        $style = 'color: ' . $searchford_option['highlight_text_color'] . ';';
                                }
                                
                                if( $searchford_option['highlight_background_color'] != '' ) {
                                        $style = 'background-color: ' . $searchford_option['highlight_background_color'] . ';';
                                }
                                if( '' != $style ) {
                                        $markup_start = '<span style="' . $style . '">';
                                        $markup_end = '</span>';
                                }
                                break;
                        case 'css':
                                if( isset( $searchford_option['highlight_css'] ) ) {
                                        $markup_start = '<span style="' . $searchford_option['highlight_css'] . '">';
                                        $markup_end = '</span>';
                                }
                                break;
                }
                return preg_replace('/' . $terms . '/i', $markup_start . "\$0" . $markup_end, $content);
        }
}

if( !function_exists('searchford_highlight_title') ) {
        function searchford_highlight_title( $query ) {
                
                if ( !is_admin() && $query->is_main_query() ) {
                        if ($query->is_search) {
                                foreach( $query->posts as $post ) {
                                        if( isset( $post->post_title ) ) {
                                                $post->post_title = searchford_highlight_term( $post->post_title );    
                                        }
                                } 
                        }
                }
        }
}
  
if( !function_exists('searchford_highlight_excerpt') ) {
        function searchford_highlight_excerpt( $excerpt ) {
                return searchford_highlight_term( $excerpt );
        }
}
  
if( !function_exists('searchford_highlight_category') ) {
        function searchford_highlight_category( $html ) {
                return searchford_highlight_term( $html );
        }
}

if( !function_exists('searchford_highlight_tags') ) {
        function searchford_highlight_tags( $url, $term ) {
                $term->name = searchford_highlight_term( $term->name );
                return $url;
        }
}

if( !function_exists('searchford_highlight_author') ) {
        function searchford_highlight_author( $name ) {
                return searchford_highlight_term( $name );
        }
}

if( !function_exists('searchford_template') ) {
        function searchford_template( $template ) {
                if( $searchford_id = get_query_var('sf') ) {
                        $searchford_option = get_post_meta( $searchford_id, '_searchford_meta', true );
                        return locate_template($searchford_option['results_page']); 
                }
                return $template;
        }
}

if( !function_exists('searchford_search_filter') ) {
        function searchford_search_filter( $query ) {
                if ( !is_admin() && $query->is_main_query() ) {
                        if ($query->is_search) {
                                
                                // Set searchford options
                                if( $searchford_id = $query->get('sf') ) {
                                        $default_options = get_option( SEARCHFORD_OPTIONS );
                                        $custom_options = get_post_meta( $searchford_id, '_searchford_meta', true );
                                        $searchford_option = array_merge( array('post_status' => $default_options['post_status']), $custom_options );
                                        if( !$custom_options ) {
                                                $searchford_option = $default_options;
                                        }
                                } else {
                                        $searchford_option = get_option( SEARCHFORD_OPTIONS );
                                }
                                
                                // Get searchford results
                                $sf_query = searchford_query( $query->query_vars['s'], $searchford_option );

                                if( count( $sf_query ) > 0 ) {
                                       
                                        $id_arr = array();
                                        $id_str = '';
                                        
                                        foreach( $sf_query as $q ) {
                                                $id_arr[] = $q['object_id'];
                                                $id_str .= ',' . $q['object_id'];
                                        }
                                        
                                        $query->set('post__in', $id_arr);

                                        if( 'date' != $searchford_option['sort'] ) {
                                                $query->set('searchford_order', trim($id_str, ','));
                                                add_filter('posts_orderby', 'searchford_orderby', 10, 2);
                                        }
                                        
                                        if( '' == $query->get('post_type') ) {
                                                $query->set('post_type', $searchford_option['include']);
                                        }
                                        
                                        if( '' == $query->get('post_status') ) {
                                                $statuses  = $searchford_option['post_status'];
                                                
                                                if( in_array('attachment', $searchford_option['include']) ) {
                                                        $statuses[] = 'inherit';
                                                }
                                                
                                                $query->set('post_status', $statuses);
                                        }
                                        
                                        // Exlude tags, categories and taxonomies
                                        if( '' != $searchford_option['exclude_category'] ){
                                                $cats = array_map( 'trim', explode(',', $searchford_option['exclude_category']) );
                                                $query->tax_query->queries[] = array(
                                                        'taxonomy' => 'category',
                                                        'field' => 'slug',
                                                        'terms' => $cats,
                                                        'operator' => 'NOT IN',
                                                );
                                        }
                                        
                                        if( '' != $searchford_option['exclude_tag'] ){
                                                $tags = array_map( 'trim', explode(',', $searchford_option['exclude_tag']) );
                                                $query->tax_query->queries[] = array(
                                                        'taxonomy' => 'post_tag',
                                                        'field' => 'slug',
                                                        'terms' => $tags,
                                                        'operator' => 'NOT IN',
                                                );
                                        }
                                        
                                        $taxonomies = get_taxonomies();
                                        unset( $taxonomies['nav_menu'], $taxonomies['post_format'], $taxonomies['post_tag'], $taxonomies['category'] );

                                        for( $i = 0; $i < count( $taxonomies ); $i++ ) {
                                                if( isset( $searchford_option['exclude_taxonomy' . $i] ) && isset( $searchford_option['exclude_taxonomy_slug' . $i] )
                                                   && '' != $searchford_option['exclude_taxonomy' . $i] && '' != $searchford_option['exclude_taxonomy_slug' . $i] ) {
                                                        $terms = array_map( 'trim', explode(',', $searchford_option['exclude_taxonomy_slug' . $i]) );
                                                        $query->tax_query->queries[] = array(
                                                                'taxonomy' => $searchford_option['exclude_taxonomy' . $i],
                                                                'field' => 'slug',
                                                                'terms' => $terms,
                                                                'operator' => 'NOT IN',
                                                        );
                                                }
                                        }
                                        
                                        $tax_query = $query->get('tax_query');
                                        if( count( $tax_query ) > 1 ){
                                                $query->tax_query->queries['relation'] = 'AND';
                                                
                                        }
                                        $query->query_vars['tax_query'] = $query->tax_query->queries;
                                        
                                        // Highlight Search Terms
                                        if( $searchford_option['highlight_title'] == '1' && in_array('title', $searchford_option['search_in']) ) {
                                                add_action( 'loop_start', 'searchford_highlight_title' );
                                        }
                                        if( $searchford_option['highlight_excerpt'] == '1' && in_array('content', $searchford_option['search_in']) ) {
                                                add_filter( 'get_the_excerpt', 'searchford_highlight_excerpt' );
                                        }
                                        if( $searchford_option['highlight_tags'] == '1'  && ( in_array('post_tag', $searchford_option['search_in']) ||
                                            in_array('category', $searchford_option['search_in']) || in_array('taxonomy', $searchford_option['search_in']) ) ) {
                                                add_filter( 'the_category', 'searchford_highlight_category' );
                                                add_filter( 'term_link', 'searchford_highlight_tags', 10, 2 );
                                        }
                                        if( $searchford_option['highlight_author'] == '1'  && in_array('author', $searchford_option['search_in']) ) {
                                                add_filter( 'the_author', 'searchford_highlight_author' );
                                        }
                                        
                                        add_filter('posts_search', 'searchford_where');

                                } else {
                                        $query->set('post__in', array(0));
                                }
                                
                                if( isset( $searchford_option['results_page']) && $searchford_option['results_page'] != '' ) {
                                        add_filter('template_include', 'searchford_template');
                                }
                        }
                }
        }
}

add_action('pre_get_posts','searchford_search_filter');

/* ==========================================================================
   Searchford Query
   ========================================================================== */
if( !function_exists('searchford_query') ) {
        function searchford_query( $term, $searchford_options ) {
                
                global $wpdb;
                
                $search_in = $searchford_options['search_in'];
                $table = $wpdb->prefix . 'searchford';
                
                $relevance = "";
                $where = "";
                
                if( count( $search_in ) > 0 ) {
                        foreach( $search_in as $si ) {
                                if( $si == 'title' || $si == 'content' ) {
                                        $where .= "subtype = '$si' OR ";
                                        $relevance .= "(MATCH(subtype) AGAINST('$si' IN BOOLEAN MODE))*" . $searchford_options[$si . '_weight'] . " +";
                                }
                                if( $si == 'attachment' || $si == 'category' || $si == 'post_tag' || $si == 'comment' ) {
                                        $where .= "type = '$si' OR ";
                                        $relevance .= "(MATCH(type) AGAINST('$si' IN BOOLEAN MODE))*" . $searchford_options[$si . '_weight'] . " +";
                                }
                                if( $si == 'taxonomy' ) {
                                        $relevance .= "(MATCH(type) AGAINST('$si' IN BOOLEAN MODE))*" . $searchford_options[$si . '_weight'] . " +";
                                        if( isset( $searchford_options['custom_taxonomy'] ) ) {
                                                foreach( $searchford_options['custom_taxonomy'] as $ct ) {
                                                        $where .= "( type = '$si' AND post_status = '$ct' ) OR ";
                                                }
                                        }
                                }
                                if( $si == 'author' ) {
                                        $relevance .= "(MATCH(type) AGAINST('$si' IN BOOLEAN MODE))*" . $searchford_options[$si . '_weight'] . " +";
                                        if( isset( $searchford_options['roles'] ) ) {
                                                foreach( $searchford_options['roles'] as $role ) {
                                                        $where .= "( type = '$si' AND subtype = '$role' ) OR ";
                                                }
                                        }
                                }
                                if( $si == 'meta' ) {
                                        $relevance .= "(MATCH(type) AGAINST('$si' IN BOOLEAN MODE))*" . $searchford_options[$si . '_weight'] . " +";
                                        $cms = array_map( 'trim', explode(',', $searchford_options['custom_meta']) );
                                        foreach( $cms as $cm ) {
                                                $where .= "( type = '$si' AND subtype LIKE '$cm' ) OR ";
                                        }
                                }
                        }  
                }

                $relevance = rtrim($relevance, ' +');
                if( $relevance == "" ) {
                        $select = "";
                        $orderby = "ORDER BY table1.object_id DESC";
                } else {
                        $select = ", table2.rel";
                        $relevance = ", ( $relevance ) AS rel";
                        $orderby = "ORDER BY table2.rel DESC, table1.object_id DESC";
                }
                
                $where = rtrim($where, ' OR ');
                if( $where == "" ) {
                        $where = "";
                } else {
                        $where = "WHERE $where";
                }
                
                $stopwords = array_map( 'trim', explode(',', $searchford_options['stopwords']) );
                $term = trim( str_replace($stopwords, '', $term) );

                if( '' == $term ) {
                        return;
                } else {
                        $term = searchford_contains_CJK( $term ) ? searchford_utf2ucs( $term, false ) : $term; //convert CJK characters to ucs
                        $term = trim( preg_replace('!\s+!', ' ', $term) ); //remove multiple whitespace
                        $operators_exist = preg_match('[\+|\-|<|>|~|"|\*|\(|\)]', $term);

                        if( $searchford_options['fuzzy'] == '1' && !$operators_exist ) {
                                $columns = "text,text_reverse";
                                $term = SEARCHFORD_PREFIX . str_replace(' ', '* ' . SEARCHFORD_PREFIX, $term ) . '* ' . str_replace(' ', '* ', searchford_reverse( SEARCHFORD_PREFIX . str_replace(' ', ' ' . SEARCHFORD_PREFIX, $term ) )) . '*';
                                $stopwords_str = $searchford_options['stopwords'] == '' ? '' : SEARCHFORD_PREFIX . str_replace(',', '* ' . SEARCHFORD_PREFIX, trim( preg_replace('!\s+!', '', $searchford_options['stopwords']))) . '*';
                                
                        } else {
                                $columns = "text";
                                $term = preg_replace('/(\s(?=[^\+\-\'<>~\(\)\\\"])|\((?=[^\+\-<>~\s\(\)\\\"\\\'])|\)(?=[^\+\-<>~\s\(\)\\\"\\\'])|"(?=[^\+\-<>~\s\(\)])|\+|\-|<|>|~)+/', '$1' . SEARCHFORD_PREFIX, $term); //add prefix
                                $stopwords_str = $searchford_options['stopwords'] == '' ? '' : SEARCHFORD_PREFIX . str_replace(',', ' ' . SEARCHFORD_PREFIX, trim( preg_replace('!\s+!', '', $searchford_options['stopwords'])));
                                preg_match_all('/-(.*?)(?=\s|$)/',$term,$exclude);

                                if( $term[0] != '+' && $term[0] != '-' && $term[0] != '<' && $term[0] != '>' && $term[0] != '\\' &&
                                    $term[0] != '~' && $term[0] != '"' && $term[0] != '*' && $term[0] != '(' && $term[0] != ')') {
                                        $term = SEARCHFORD_PREFIX . $term;
                                }
                        }

                        $query = "SELECT table1.object_id$select
                                  FROM $table AS table1
                                  RIGHT JOIN (
                                        SELECT id$relevance
                                        FROM $table
                                        WHERE
                                            MATCH($columns) AGAINST('$term' IN BOOLEAN MODE)
                                        GROUP BY id
                                  ) AS table2 ON table1.id = table2.id
                                  $where
                                  GROUP BY table1.object_id
                                  $orderby";
                                  
                        $results = $wpdb->get_results($query, ARRAY_A);

                        // Exclude terms and stopwords
                        $exclude_terms = isset( $exclude[1] ) ? trim( implode(' ', $exclude[1]) ) : '';
                        $exclude_str = $exclude_terms == '' ? $exclude_terms . ' ' . $stopwords_str : $stopwords_str;

                        if( $exclude_str != '' ) {
                                $exclude_results = $wpdb->get_results(
                                                "SELECT object_id
                                                FROM $table
                                                WHERE
                                                    MATCH(text) AGAINST('$exclude_str' IN BOOLEAN MODE)
                                                GROUP BY object_id"
                                                , ARRAY_A);
                                        
                                        $exclude_array = array();
                                        foreach( $exclude_results as $er ) {
                                                $exclude_array[] = $er['object_id'];
                                        }
                                        
                                        foreach( $results as $k => $result ) {
                                                if( in_array( $result['object_id'], $exclude_array ) ) {
                                                        unset( $results[$k] );
                                                }
                                        }
                        }
                        return $results;
                }
        }
}

/* ==========================================================================
   Search Suggestions
   ========================================================================== */
if( !function_exists('searchford_get_suggestion') ) {
        function searchford_get_suggestion( $term, $searchford_options ) {
        
                $match = '';
                $terms = trim( str_replace(array('+','(',')','>','<','~','*','"'), '', $term) ); //remove operators
                
                if( strlen($terms) == strlen(utf8_decode($terms)) ) { //check if language supports levenshtein
                        global $wpdb;
               
                        $table = $wpdb->prefix . 'searchford';
                        $where = '';
                        $stopwords = explode(',', str_replace(' ', '', $searchford_options['stopwords']));
                        foreach( $stopwords as $k => $sw ) {
                                $stopwords[$k] = SEARCHFORD_PREFIX . $sw;
                        }
                        foreach( $searchford_options['search_in'] as $si ) {
                        
                                if( $si == 'title' || $si == 'content' ) {
                                        $where .= "subtype = '$si' OR ";
                                }
                                if( $si == 'attachment' || $si == 'category' || $si == 'post_tag' || $si == 'comment' ) {
                                        $where .= "type = '$si' OR ";
                                }
                                if( $si == 'taxonomy' ) {
                                        if( isset( $searchford_options['custom_taxonomy'] ) ) {
                                                foreach( $searchford_options['custom_taxonomy'] as $ct ) {
                                                        $where .= "( type = '$si' AND post_status = '$ct' ) OR ";
                                                }
                                        }
                                }
                                if( $si == 'author' ) {
                                        if( isset( $searchford_options['roles'] ) ) {
                                                foreach( $searchford_options['roles'] as $role ) {
                                                        $where .= "( type = '$si' AND subtype = '$role' ) OR ";
                                                }
                                        }
                                }
                                if( $si == 'meta' ) {
                                        $cms = array_map( 'trim', explode(',', $searchford_options['custom_meta']) );
                                        foreach( $cms as $cm ) {
                                                $where .= "( type = '$si' AND subtype LIKE '$cm' ) OR ";
                                        }
                                }
                        }
                        
                        $where = rtrim($where, ' OR ');
                        $where = $where == "" ? "" : "WHERE $where";
                        $results = $wpdb->get_results("SELECT object_id, type, text FROM $table $where");
                        $search = explode(' ', SEARCHFORD_PREFIX . str_replace(' ', ' ' . SEARCHFORD_PREFIX, $terms));
                        $last = end($search);

                        foreach( $search as $k => $s ) {
                                foreach( $results as $r ) {
                                        
                                        if( function_exists('icl_object_id') ) {
                                                if( !in_array( $r->type, array('comment','taxonomy','author','meta')) ) {
                                                        if( $r->object_id != icl_object_id( $r->object_id, $r->type, false) ) {
                                                                break;
                                                        }
                                                }
                                        }
                                        
                                        if( strlen($r->text) == strlen(utf8_decode($r->text)) ) {
                                                $words = explode(' ', mb_strtolower($r->text));
                                                foreach( $words as $word ) {
                                                        if( !in_array( $word, $stopwords ) ) {
                                                                if( levenshtein( $s, $word ) < 2 ) {
                                                                        $match .= ' ' . $word;
                                                                        if( $s == $last ) {
                                                                                break 3;
                                                                        }
                                                                        break 2;
                                                                }
                                                        }
                                                }
                                        }
                                }
                        }
                }
                return trim( str_replace(SEARCHFORD_PREFIX, '', $match ));
        }
}
if( !function_exists('searchford_no_results') ) {
        function searchford_no_results( $posts, $wp_query ) {
                if ( !is_admin() && $wp_query->is_main_query() ) {
                        if ( $wp_query->is_search ) {
                                
                                if( $searchford_id = get_query_var('sf') ) {
                                        $searchford_option = get_post_meta( $searchford_id, '_searchford_meta', true);
                                        if( !$searchford_option ) {
                                                $searchford_option = get_option( SEARCHFORD_OPTIONS );
                                        }
                                } else {
                                        $searchford_option = get_option( SEARCHFORD_OPTIONS );
                                }

                                if( !count( $posts ) && $searchford_option['no_results'] != '0' ) {

                                        $term = $wp_query->get('s');
                                        
                                        if( '' != $term ) {
                                                
                                                $match = searchford_get_suggestion( preg_replace('/-.*?(\s|$)/','',$term), $searchford_option );
                                                
                                                if( '' != $match && $match != $term ) {
                                                        
                                                        global $searchford_suggestion;
                                                        
                                                        if( $searchford_option['no_results'] == '1' ) {
                                                                
                                                                $url = add_query_arg(array('s' => urlencode($match) ));
                                                                $searchford_suggestion = sprintf(__('Did you mean %1$s?', SEARCHFORD_SLUG), '<a href="' . $url . '">' . $match . '</a>');
                                                                
                                                        } elseif( $searchford_option['no_results'] == '2' ) {
                                                                
                                                                $url = add_query_arg(array('s' => urlencode($match), 'missed' => urlencode($term) ));
                                                                wp_redirect( $url );
                                                                exit;
                                                        }
                                                }
                                        }
                                }
                        }
                }
                return $posts;
        }
}

add_filter('the_posts','searchford_no_results', 10, 2);

/* ==========================================================================
   Searchford Ajax Actions
   ========================================================================== */
if( !function_exists('searchford_box_callback') ) {
        function searchford_box_callback() {

                $searchford_option = get_option( SEARCHFORD_OPTIONS );
                $response = array();
                
                if( $searchford_option['live'] == '1' ) {
                        $theme = $searchford_option['theme'] == 'dark' ? ' dark' : '';
                        $radius = $searchford_option['round'] == '1' ? ' radius' : '';
                        $right_tip = $searchford_option['align'] == 'right' ? ' searchford-tip-right' : '';
                        $tip = $searchford_option['search_tip'] == '1' ? '<div class="searchford-tip' . $right_tip . '"></div>' : '';
                        $align = $searchford_option['align'] == 'right' ? ' data-searchford-align="right"' : ' data-searchford-align="left"';
                        $load_content = '';
                        $load_background = '';
        
                        if( $searchford_option['loading_gif'] == '' ) {
                                $load_content = '<div class="search-loading"><div class="l1"><div></div></div><div class="l2"><div></div></div><div class="l3"><div></div></div><div class="l4"><div></div></div></div>';
                        } else {
                                $load_src = wp_get_attachment_image_src( $searchford_option['loading_gif'], 'full' );
                                $load_background = ' style="background: url(' . $load_src[0] . ') no-repeat center center;"';
                        }
                        
                        $response['html'] = '<div class="searchford-box' . $radius . $theme . '"' . $align . '>'
                              . $tip
                              . '<div class="searchford-content"></div>'
                              . '<div class="searchford-load center"' . $load_background . '>'
                              . $load_content
                              . '</div><div class="searchford-foot">' . __('SEARCHING', SEARCHFORD_SLUG) . '</div></div>';
                              
                        if( $searchford_option['live_style'] == 'masonry' ) {
                                $width = $searchford_option['width'] != '' ? (int) $searchford_option['width'] / (int) $searchford_option['masonry_columns'] . 'px' : 450 / (int) $searchford_option['masonry_columns'] . 'px';
                                $response['grid_options'] = array(
                                        'max_width' => $width,             // grid item width in pixel, default: 220px
                                        'width' => $width, 
                                );
                        }
                }
                echo json_encode( $response );
                
                die();
        }
}

if( !function_exists('searchford_ajax_query_callback') ) {
        function searchford_ajax_query_callback() {
                
                if( isset( $_POST['sf'] ) ) {
                        $default_options = get_option( SEARCHFORD_OPTIONS );
                        $searchford_option = array_merge( array('post_status' => $default_options['post_status']), get_post_meta( $_POST['sf'], '_searchford_meta', true) );
                } else {
                        $searchford_option = get_option( SEARCHFORD_OPTIONS );
                }
                
                $sf_query = searchford_query( $_POST['s'], $searchford_option );

                if( count( $sf_query ) > 0 ) {
                       
                        $arg = array();
                        $id_arr = array();
                        $id_str = '';
                        
                        foreach( $sf_query as $q ) {
                                $id_arr[] = $q['object_id'];
                                $id_str .= ',' . $q['object_id'];
                        }
                        
                        $args['post__in'] = $id_arr;
                        $args['post_type'] = $searchford_option['include'];
                        $args['posts_per_page'] = is_int( $searchford_option['live_max'] ) && $searchford_option['live_max'] > 0 ? (int) $searchford_option['live_max'] : 10;

                        $statuses = in_array('attachment', $searchford_option['include']) ? array_push($searchford_option['post_status'], 'inherit') : $searchford_option['post_status'];
                        $args['post_status'] = $statuses;
                        
                        if( 'date' != $searchford_option['sort'] ) {
                                $args['searchford_order'] = trim($id_str, ',');
                        }
                        
                        // Exlude tags, categories and taxonomies
                        $tax_arr = array();
                        
                        if( '' != $searchford_option['exclude_category'] ){
                                $cats = array_map( 'trim', explode(',', $searchford_option['exclude_category']) );
                                $tax_arr[] = array(
                                        'taxonomy' => 'category',
                                        'field' => 'slug',
                                        'terms' => $cats,
                                        'operator' => 'NOT IN',
                                );
                        }
                        
                        if( '' != $searchford_option['exclude_tag'] ){
                                $tags = array_map( 'trim', explode(',', $searchford_option['exclude_tag']) );
                                $tax_arr[] = array(
                                        'taxonomy' => 'post_tag',
                                        'field' => 'slug',
                                        'terms' => $tags,
                                        'operator' => 'NOT IN',
                                );
                        }
                        
                        $taxonomies = get_taxonomies();
                        unset( $taxonomies['nav_menu'], $taxonomies['post_format'], $taxonomies['post_tag'], $taxonomies['category'] );

                        for( $i = 0; $i < count( $taxonomies ); $i++ ) {
                                if( isset( $searchford_option['exclude_taxonomy' . $i] ) && isset( $searchford_option['exclude_taxonomy_slug' . $i] )
                                   && '' != $searchford_option['exclude_taxonomy' . $i] && '' != $searchford_option['exclude_taxonomy_slug' . $i] ) {
                                        $terms = array_map( 'trim', explode(',', $searchford_option['exclude_taxonomy_slug' . $i]) );
                                        $tax_arr[] = array(
                                                'taxonomy' => $searchford_option['exclude_taxonomy' . $i],
                                                'field' => 'slug',
                                                'terms' => $terms,
                                                'operator' => 'NOT IN',
                                        );
                                }
                        }
                        
                        if( count( $tax_arr ) > 1 ){
                                $tax_arr['relation'] = 'AND';
                                
                        }
                        $args['tax_query'] = $tax_arr;
                        $the_query = new WP_Query( $args );
                        
                        add_filter('posts_orderby', 'searchford_orderby', 10, 2);
                        add_filter('posts_search', 'searchford_where');
                        
                        $the_query->query($args);
                        if( $the_query->post_count > 0 ) {
                                
                                if( $searchford_option['live_style'] == 'masonry' ) {
                                        $masonry = '';
                                        $columns = isset( $searchford_option['masonry_columns'] ) && is_numeric( $searchford_option['masonry_columns'] ) ? $searchford_option['masonry_columns'] : 3;
                                        foreach( $the_query->posts as $post ) {
                                                if( has_post_thumbnail( $post->ID ) || wp_attachment_is_image( $post->ID ) ) {
                                                        
                                                        $masonry_title = '';
                                                        $with_title = '';
                                                        if( $searchford_option['display_title'] == 1 && $post->post_title != '' ) {
                                                                $masonry_title = '<p class="searchford-masonry-title searchford-title">' . esc_html( $post->post_title ) . '</p>';
                                                                $with_title = ' class="with-title"';
                                                        }
                                                        $masonry .= wp_attachment_is_image( $post->ID ) ? '<li><a' . $with_title . ' href="' . get_permalink( $post->ID ) . '">' .  wp_get_attachment_image( $post->ID, 'thumbnail' ) . $masonry_title . '</a></li>' : '<li><a' . $with_title . ' href="' . get_permalink( $post->ID ) . '">' .  get_the_post_thumbnail( $post->ID, 'thumbnail' ) . $masonry_title . '</a></li>';
                                                }
                                        }
                                        $html = $masonry == '' ? '' : '<div class="searchford-masonry-container"><ul class="searchford-masonry small-block-grid-' . $columns . '">' . $masonry . '</ul></div>';
                                } else {
                                        
                                        $content = '';
                                        
                                        if( ( $searchford_option['display_thumbnail'] == 0 && $searchford_option['display_title'] == 0 && $searchford_option['display_excerpt'] == 0 ) ||
                                            ( $searchford_option['display_thumbnail'] == 0 && $searchford_option['display_title'] == 1 && $searchford_option['display_excerpt'] == 0 )) {
                                                foreach( $the_query->posts as $post ) {
                                                        $content .= $post->post_title != '' ? '<li><a href="' . get_permalink( $post->ID ) . '"><div class="searchford-title">' . esc_attr( $post->post_title ) . '</div></a></li>' : '';
                                                }
                                                
                                        } elseif( $searchford_option['display_thumbnail'] == 1 && $searchford_option['display_title'] == 0 && $searchford_option['display_excerpt'] == 0 ) {
                                                foreach( $the_query->posts as $post ) {
                                                        if( has_post_thumbnail( $post->ID ) || wp_attachment_is_image( $post->ID ) ) {
                                                                $content .= wp_attachment_is_image( $post->ID ) ? '<li><a href="' . get_permalink( $post->ID ) . '">' .  wp_get_attachment_image( $post->ID, 'thumbnail' ) . '</a></li>' : '<li><a href="' . get_permalink( $post->ID ) . '">' .  get_the_post_thumbnail( $post->ID, 'thumbnail' ) . '</a></li>';
                                                        }
                                                }
                                                
                                        } else {
                                                
                                                foreach( $the_query->posts as $post ) {
                                                                
                                                        $expt = $post->post_excerpt == '' ? searchford_clean_string( $post->post_content ) : $post->post_excerpt;
                                                        $no_title = $searchford_option['display_title'] == 1 && $post->post_title != '' ? '' : ' no-title';
                                                        $title_only = ( $searchford_option['display_title'] == 1 && $post->post_title != '' ) && ( !$searchford_option['display_excerpt'] == 1 || $expt == '' ) ? ' title-only' : '';
                                                        $with_excerpt = $searchford_option['display_excerpt'] == 1 ? ' with-excerpt' : '';
                                                
                                                        $content .= '<li><a href="' . get_permalink( $post->ID ) . '"><table><tr>';
                                                        $content .= $searchford_option['display_thumbnail'] == 1 && has_post_thumbnail( $post->ID ) ? '<td class="searchford-thumb">' .  get_the_post_thumbnail( $post->ID, 'thumbnail' ) . '</td><td class="searchford-text' . $title_only . '">' : '<td class="searchford-text no-thumbnail' . $title_only . '">';
                                                        $content .= $searchford_option['display_title'] == 1 && $post->post_title != '' ? '<div class="searchford-title' . $with_excerpt . '">' . esc_attr( $post->post_title ) . '</div>' : '';
                                                        $content .= $searchford_option['display_excerpt'] == 1 && $expt != '' ? '<div class="searchford-excerpt' . $no_title . '">' . $expt . '</div>' : '';
                                                        $content .= '</td></tr></table></a></li>';
                                                }
                                        }
                                        
                                        $html = $content == '' ? '' : '<ul class="searchford-list">' . $content . '</ul>';
                                }
                                
                                if( $html != '' ) {
                                        wp_reset_postdata();
                                        echo json_encode( array('results' => $html, 'style' => $searchford_option['live_style'] ) );
                                        die();  
                                }
                        }
                }
                wp_reset_postdata();
                
                if( $searchford_option['live_suggestion'] == '1' ) {
                                
                        $match = searchford_get_suggestion( preg_replace('/-.*?(\s|$)/','',$_POST['s']), $searchford_option );
                                        
                        if( '' != $match && $match != $_POST['s'] ) {
                                
                                $vars = $_POST;
                                $vars['s'] = urlencode($match);
                                unset($vars['action']);
                                $url = add_query_arg($vars, get_home_url());
                                echo json_encode(array('suggestion' => sprintf(__('Did you mean %1$s?', SEARCHFORD_SLUG), '<a href="' . $url . '">' . $match . '</a>')));
                                die();
                        }
                }
                
                echo json_encode('no_results');
                die();
        }
}

add_action( 'wp_ajax_nopriv_searchford_box', 'searchford_box_callback' );
add_action( 'wp_ajax_searchford_box', 'searchford_box_callback' );
add_action( 'wp_ajax_nopriv_searchford_ajax_query', 'searchford_ajax_query_callback' );
add_action( 'wp_ajax_searchford_ajax_query', 'searchford_ajax_query_callback' );

/* ==========================================================================
   Searchford Suggestion Shortcode
   ========================================================================== */
if (!function_exists('searchford_suggestion')) {
        
        function searchford_suggestion( $attr ) {
                
                extract(shortcode_atts(array(
                        'tag'           => '',
                        'class'        => '',
                ), $attr));
                
                global $searchford_suggestion;
                
                $html = '';
                $classes = $class != '' ? ' class="' . esc_attr( $class ) . '"' : '';
                $open_tag = $tag != '' ? '<' . $tag . $classes . '>' : '';
                $close_tag = $tag != '' ? '</' . $tag . '>' : '';
                
                if( $searchford_suggestion ) {
                        $html = $open_tag . $searchford_suggestion . $close_tag;
                } else {

                        parse_str(parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY), $query);
                        if( isset( $query['missed'] ) && $s = get_query_var('s') ) {
                                $url = '<a href="' . add_query_arg(array('missed' => false )) . '">' . $s . '</a>';
                                $html = $open_tag . sprintf(__('No results found for %1$s. Did you mean %2$s?', SEARCHFORD_SLUG), urldecode( $query['missed'] ), $url) . $close_tag;
                        }
                }
                
                return $html;
        }
}
add_shortcode("searchford_suggestion", "searchford_suggestion");

/* ==========================================================================
   Searchford Shortcode
   ========================================================================== */
if (!function_exists('searchford_shortcode')) {
        
        function searchford_shortcode( $attr ) {
                
                extract(shortcode_atts(array(
                        'id'           => '',
                ), $attr));
                
                $custom = get_post_meta( $id, '_searchford_meta', true);
                
                if( !$custom ) {
                        return;
                }
                
                // get custom settings
                $live = $custom['live'] == '1' ? ' searchford-live' : '';
                $autocomplete = $custom['live'] == '1' ? ' autocomplete="off"' : '';
                $submit = $custom['submit'] == '1' && $custom['submit_label'] != '' ? '<input type="submit" id="searchsubmit" value="' . $custom['submit_label'] . '" />' : '';
                $live_html = '';
                $options = '';
                $lang = defined('ICL_LANGUAGE_CODE') ? '<input type="hidden" name="lang" value="' . ICL_LANGUAGE_CODE . '"/>' : '';
                
                if( function_exists('pll_home_url') ) {
                        $action = pll_home_url();
                } elseif( function_exists('icl_get_home_url') ) {
                        $action = icl_get_home_url();
                } else {
                        $action = esc_url( home_url( '/' ) );
                }
                
                if( $custom['live'] == '1' ) {
                        $theme = $custom['theme'] == 'dark' ? ' dark' : '';
                        $radius = $custom['round'] == '1' ? ' radius' : '';
                        $right_tip = $custom['align'] == 'right' ? ' searchford-tip-right' : '';
                        $tip = $custom['search_tip'] == '1' ? '<div class="searchford-tip' . $right_tip . '"></div>' : '';
                        $align = $custom['align'] == 'right' ? ' data-searchford-align="right"' : ' data-searchford-align="left"';
                        $load_content = '';
                        $load_background = '';
        
                        if( $custom['loading_gif'] == '' ) {
                                $load_content = '<div class="search-loading"><div class="l1"><div></div></div><div class="l2"><div></div></div><div class="l3"><div></div></div><div class="l4"><div></div></div></div>';
                        } else {
                                $load_src = wp_get_attachment_image_src( $custom['loading_gif'], 'full' );
                                $load_background = ' style="background: url(' . $load_src[0] . ') no-repeat center center;"';
                        }
                        
                        $live_html = '<div id="searchford_box' . (int) $id . '" class="searchford-box' . $radius . $theme . '"' . $align . '>'
                                . $tip
                                . '<div class="searchford-content"></div>'
                                . '<div class="searchford-load center"' . $load_background . '>'
                                . $load_content
                                . '</div><div class="searchford-foot">' . __('SEARCHING', SEARCHFORD_SLUG) . '</div></div>';
                                
                              
                        if( $custom['live_style'] == 'masonry' ) {
                                $width = $custom['width'] != '' ? (int) $custom['width'] / (int) $custom['masonry_columns'] . 'px' : 450 / (int) $custom['masonry_columns'] . 'px';
                                $options = " data-searchford='{\"width\":\"$width\",\"max_width\":\"$width\"}'";
                        }
                }
                
                // generate html
                ob_start();                      // start capturing output
                include( SEARCHFORD_PATH . '/public/partials/searchford-public-searchform.php');   // execute the file
                $html = ob_get_contents();    // get the contents from the buffer
                ob_end_clean();

                return $html;
        }
}
add_shortcode("searchford", "searchford_shortcode");

/* ==========================================================================
   Searchford Widget
   ========================================================================== */
class searchford_widget extends WP_Widget { 
	
	// Widget Settings
	function __construct() {
		parent::__construct(
			'searchford_widget', // Base ID
			__('Searchford Custom Search', SEARCHFORD_SLUG), // Name
			array( 'description' => __( 'Displays custom search form in the sidebar', SEARCHFORD_SLUG ), ) // Args
		);
	}
	
	// Widget Output
	function widget($args, $instance) {
                
                extract( $args );
		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $before_widget;
		if ( ! empty( $title ) ) echo $before_title . $title . $after_title;
                
                if( get_post_status( $instance['searchford'] ) == 'publish' && get_post_type( $instance['searchford'] ) == 'searchford' ) {
                       echo do_shortcode('[searchford id="' . $instance['searchford'] . '" style="_widget"]'); 
                }
                
		echo $after_widget;
	}
	
	// Update
	function update( $new_instance, $old_instance ) {  
		$instance = $old_instance; 
                
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
                $instance['searchford'] = (int) $new_instance['searchford'];

		return $instance;
	}
	
	// Backend Form
	function form($instance) {
		
		$defaults = array('title' => '', 'searchford' => '',);
		$instance = wp_parse_args((array) $instance, $defaults);
                
                $args= array(
			'post_type' => 'searchford',
			'posts_per_page' => -1,
                        'post_status' => 'publish',
		);
                $posts = new WP_Query($args);
                ?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>">Title:</label>
			<input type="text" class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php echo $instance['title']; ?>" />
		</p>
                <p>
			<label for="<?php echo $this->get_field_id('searchford'); ?>"><?php _e('Searchford:',SEARCHFORD_SLUG); ?></label>
                        <select id="<?php echo $this->get_field_id('searchford'); ?>" name="<?php echo $this->get_field_name('searchford'); ?>" class="widefat">  
                                <option><?php _e('Choose custom search',SEARCHFORD_SLUG); ?></option>
                                <?php while ( $posts->have_posts() ) : $posts->the_post(); ?>
                                <option value="<?php the_ID(); ?>" <?php selected( $instance['searchford'], get_the_ID()); ?>><?php the_title(); ?></option>
                                <?php endwhile; ?>
                        </select>
		</p>
                <p>
                        <a href="<?php echo get_option("siteurl"); ?>/wp-admin/post-new.php?post_type=searchford" target="_blank"><?php _e('Add new custom search', SEARCHFORD_SLUG); ?></a>
                </p>
		
        <?php }
}

// Add Widget
function searchford_widget_init() {
	register_widget('searchford_widget');
}
add_action('widgets_init', 'searchford_widget_init');