<?php
/*
 * Plugin Name: Searchford WordPress Advanced Search (shared on wplocker.com)
 * Plugin URI: http://archtheme.com/searchford
 * Description: Advanced WordPress Search Plugin. Boost in-site search immediately and add live search previews.
 * Author: ArchTheme
 * Author URI: http://archtheme.com/searchford
 * Version: 1.0.6
 */

// Define plugin path
if ( !defined('SEARCHFORD_PATH') ) {
       define('SEARCHFORD_PATH', plugin_dir_path(__FILE__) ); 
}

// Define plugin url
if ( !defined('SEARCHFORD_URL') ) {
       define('SEARCHFORD_URL', plugins_url( '/', __FILE__) ); 
}

// Define plugin name
if ( !defined('SEARCHFORD_NAME') ) {
        define('SEARCHFORD_NAME', "Searchford WordPress Advanced Search");
}

// Define plugin version
if ( !defined('SEARCHFORD_VERSION') ) {
        define ("SEARCHFORD_VERSION", "1.0.6");
}

// Define plugin slug
if ( !defined('SEARCHFORD_SLUG') ) {
      define ("SEARCHFORD_SLUG", 'searchford');  
}

// Define plugin options name
if ( !defined('SEARCHFORD_OPTIONS') ) {
      define ("SEARCHFORD_OPTIONS", 'searchford_options');  
}

// Define index words prefix
if ( !defined('SEARCHFORD_PREFIX') ) {
        define ("SEARCHFORD_PREFIX", '_');  
}
/* ==========================================================================
   Internalisation and Translation
   ========================================================================== */
if (!function_exists('searchford_load_language'))
{
	function searchford_load_language()
	{
		load_plugin_textdomain( SEARCHFORD_SLUG, false, dirname( plugin_basename( __FILE__ ) ) . '/languages');
	}
}

add_action('plugins_loaded', 'searchford_load_language');

/* ==========================================================================
   Default Settings
   ========================================================================== */
if( !function_exists( 'searchford_get_default_options' ) ) {
        function searchford_get_default_options() {
                
                $options = get_option( SEARCHFORD_OPTIONS );
                if ( !is_array( $options ) ) {
                        $options = array();
                }
                
                $default_options = array(
                        'sort' => 'relevance',
                        'search_in' => array('title','content','category','post_tag','comment'),
                        'roles' => array(),
                        'custom_meta' => '',
                        'custom_taxonomy' => array(),
                        'non_public' => '0',
                        'include' => array('post','page'),
                        'exclude_category' => '',
                        'exclude_tag' => '',
                        'exclude_post' => '',
                        'exclude_taxonomy_id0' => '',
                        'stopwords' => '',
                        'fuzzy' => '1',
                        'post_status' => array('publish'),
                        'no_results' => '1',
                        'title_weight' => '3',
                        'content_weight' => '2',
                        'category_weight' => '1',
                        'post_tag_weight' => '1',
                        'comment_weight' => '0.5',
                        'attachment_weight' => '1',
                        'taxonomy_weight' => '1',
                        'meta_weight' => '1',
                        'highlight_title' => '1',
                        'highlight_excerpt' => '1',
                        'highlight_tags' => '1',
                        'highlight_author' => '1',
                        'highlight_style' => 'bold',
                        'highlight_text_color' => '',
                        'highlight_background_color' => '',
                        'highlight_css' => '',
                        'index' => 'auto',
                        'live' => '1',
                        'live_style' => 'list',
                        'live_suggestion' => '1',
                        'display_thumbnail' => '1',
                        'display_title' => '1',
                        'display_excerpt' => '',
                        'live_max' => '10',
                        'masonry_columns' => '3',
                        'theme' => 'light',
                        'font_color' => '',
                        'background_color' => '',
                        'accent_color' => '',
                        'round' => '1',
                        'width' => '',
                        'search_tip' => '1',
                        'align' => 'left',
                        'loading_gif' => '',
                        'css' => '',
                        'custom_css' => '',
                        'remove_data' => '0',
                );
                
                return array_merge( $default_options, $options );
        }
}

/* ==========================================================================
   Activation and Uninstall Hooks
   ========================================================================== */
if( !function_exists('searchford_create_table') )
{
        function searchford_create_table($table, $charset_collate) {
                
                $sql = "DROP TABLE IF EXISTS $table;
                        CREATE TABLE IF NOT EXISTS $table (
                        id mediumint(9) NOT NULL AUTO_INCREMENT,
                        object_id bigint(20),
                        post_status varchar(20),
                        type varchar(255),
                        subtype varchar(255),
                        text text,
                        text_reverse text,
                        FULLTEXT (text, text_reverse),
                        PRIMARY KEY (id)
                ) $charset_collate ENGINE=MyISAM;";
        
                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                dbDelta( $sql );
        }
}

if( !function_exists('_searchford_activate') )
{
        function _searchford_activate() {
                
                global $wpdb;
                
                // If previous versions exist, regenerate custom.css
                if( get_option( 'searchford_version' ) ) {
                        searchford_update_static_css();
                }
                
                // Save plugin version in db
                add_option( 'searchford_version', SEARCHFORD_VERSION );

                // Save default options
                update_option( SEARCHFORD_OPTIONS, searchford_get_default_options() );
                
                // Create searchford index table if not exists
                $table = $wpdb->prefix . 'searchford';
                $charset_collate = $wpdb->get_charset_collate();
                
                if ($wpdb->get_var( "SHOW TABLES LIKE '{$table}'") != $table) {
                        searchford_create_table($table, $charset_collate);
                }
                
                searchford_create_index($table);
        }
}

if( !function_exists('_searchford_uninstall') )
{
        function _searchford_uninstall() {

                global $wpdb;
                
                // Drop searchford index table if exists
                $table = $wpdb->prefix . 'searchford';
                $wpdb->query("DROP TABLE IF EXISTS $table;");
                $searchford_options = get_option( SEARCHFORD_OPTIONS );
                
                if( $searchford_options['remove_data'] == '1' ) {
                        
                        // Remove searchford posts
                        $posts = get_posts(array(
                                'post_type' => 'searchford',
                                'posts_per_page' => -1,
                        ));
                        if( is_array( $posts ) ) {
                                foreach( $posts as $p ) {
                                        wp_delete_post( $p->ID, true);
                                }
                        }
                        
                        // Remove searchford options
                        delete_option( SEARCHFORD_OPTIONS );
                        delete_option('searchford_version');
                }
        }
}

if( !function_exists('searchford_network_propagate') )
{
        function searchford_network_propagate($function, $networkwide) {
                
                global $wpdb;
                
                // Multisite activation
                if (function_exists('is_multisite') && is_multisite()) {
                        // check if it is a network activation - if so, run the activation function for each blog id
                        if ($networkwide) {
                                $old_blog = $wpdb->blogid;
                                // Get all blog ids
                                $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");
                                
                                foreach ($blogids as $blog_id) {
                                        switch_to_blog($blog_id);
                                        call_user_func($function, $networkwide);
                                }
                                
                                switch_to_blog($old_blog);
                                return;
                        }   
                } 
                
                // Single site activation
                call_user_func($function, $networkwide);
        }
}

if( !function_exists('searchford_activate') )
{
        function searchford_activate( $networkwide ) {
                // Network propagate
                searchford_network_propagate('_searchford_activate', $networkwide);
        }
}

if( !function_exists('searchford_uninstall') )
{
        function searchford_uninstall( $networkwide ) {
                // Network propagate
                searchford_network_propagate('_searchford_uninstall', $networkwide);
        }
}

register_activation_hook(__FILE__, 'searchford_activate');
register_uninstall_hook(__FILE__, 'searchford_uninstall');


/* ==========================================================================
   Includes
   ========================================================================== */
include_once( SEARCHFORD_PATH . 'admin/searchford-admin.php'); //admin
include_once( SEARCHFORD_PATH . 'public/searchford-public.php'); //public
