<?php

/**
 * Handles export.
 * 
 */
if (!defined('ABSPATH')) {
    exit;
}


if (!class_exists('XA_Migration_Batch_Exporter', false)) {
    include_once( XA_MIGRATION_MAIN_PATH . '/exporter/abstract-migration-batch-exporter.php' );
}

class XA_Wp_Migration_Exporter extends XA_Migration_Batch_Exporter {

    protected $migration_zip_path;

    public function __construct() {
        parent::__construct();        
        $this->migration_zip_path = plugin_dir_path(__FILE__) . '../../../xa-migration-export/'; // wp content path - fixed WP violation
        
    }

    public function generate_zip_file() {
        $current_step = $this->get_page();
        switch ($current_step) {
            case 1:
                $this->generate_plugins_zip(); //generate plugins zip
                break;
            case 2:
                $this->generate_themes_zip(); //generate themes zip
                break;
            case 3:
                $this->generate_uploads_zip(); //generate uploads zip
                break;
            case 4:
                //$this->generate_database_sql(); //generate datbase export sql
                global $wpdb;
                if ( empty( $wpdb->use_mysqli ) ) {
                    $this->generate_database_mysql_standard(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
                }else{
                    $this->generate_database_mysqli_standard(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
                }
                break;
            default:
                break;
        }
    }

    protected function generate_plugins_zip() {


        if (file_exists($this->migration_zip_path)) {
            $this->rrmdir($this->migration_zip_path, 0777, true);
        } else {
            mkdir($this->migration_zip_path, 0777, true);
        }

        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
            $zip = new Zip();
            $zip->zip_start($this->migration_zip_path . "plugins.zip");
            $zip->zip_add(array(XA_MIGRATION_MAIN_PATH . "../../plugins"));
            $zip_res = $zip->zip_end();
        } else {
            $this->Zippp(XA_MIGRATION_MAIN_PATH . "../../plugins/", $this->migration_zip_path . "plugins.zip");
        }
    }

    protected function generate_themes_zip() {


        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
            $zip = new Zip();
            $zip->zip_start($this->migration_zip_path . "themes.zip");
            $zip->zip_add(array(XA_MIGRATION_MAIN_PATH . "../../themes"));
            $zip_res = $zip->zip_end();
        } else {
            $this->Zippp(XA_MIGRATION_MAIN_PATH . "../../themes/", $this->migration_zip_path . "themes.zip");
        }
    }

    protected function generate_uploads_zip() {

        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
            $zip = new Zip();
            $zip->zip_start($this->migration_zip_path . "uploads.zip");
            $zip->zip_add(array(XA_MIGRATION_MAIN_PATH . "../../uploads"));
            $zip_res = $zip->zip_end();
        } else {
            $this->Zippp(XA_MIGRATION_MAIN_PATH . "../../uploads/", $this->migration_zip_path . "uploads.zip");
        }
    }

    protected function generate_database_sql() {


        //require_once( ABSPATH . '/wp-load.php');
        global $wpdb;
        
        $sql_header  = "/* XA MYSQL SCRIPT CREATED ON : ".@date("Y-m-d H:i:s")." */\n\n";
	$sql_header .= "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;\n\n";
        $sql_header .= "SET FOREIGN_KEY_CHECKS = 0;\n\n";
        // Get a list of the tables
        $tables = $wpdb->get_results('SHOW TABLES');

        $file_path = $this->migration_zip_path . 'database.sql';
        $file = fopen($file_path, 'w');
        
        fwrite($file, $sql_header);

        $dbname = $wpdb->dbname;
        $tables_in = 'Tables_in_' . $dbname;

        foreach ($tables as $table) {

            $table_name = $table->{$tables_in};

            $schema = $wpdb->get_row('SHOW CREATE TABLE ' . $table_name, ARRAY_A);
            
            $table_struct = str_replace($wpdb->prefix, 'xadbprefix_', $schema['Create Table']);
            
            fwrite($file, $table_struct . ';' . PHP_EOL);

            $rows = $wpdb->get_results('SELECT * FROM ' . $table_name, ARRAY_A);

            $moving_table_name = str_replace($wpdb->prefix, 'xadbprefix_', $table_name);
            
            if ($rows) {
                fwrite($file, 'INSERT INTO ' . $moving_table_name . ' VALUES ');

                $total_rows = count($rows);
                $counter = 1;
                foreach ($rows as $row => $fields) {
                    $line = '';
                    foreach ($fields as $key => $value) {
                        $value = addslashes($value);
                        $line .= '"' . $value . '",';
                    }

                    $line = '(' . rtrim($line, ',') . ')';

                    if ($counter != $total_rows) {
                        $line .= ',' . PHP_EOL;
                    }
                    if ($line && !empty($this->find_replace)) {
                        $line = str_ireplace($this->find_replace['options']['replace']['find_value'], $this->find_replace['options']['replace']['replace_value'], $line);
                    }
                    fwrite($file, $line);

                    $counter++;
                }

                fwrite($file, '; ' . PHP_EOL);
            }
        }

        fclose($file);
    }

    
    
    
    
    public function generate_database_mysqli_standard($host, $user, $pass, $name, $tables = false, $backup_name = false) {
        
        //require_once( ABSPATH . '/wp-load.php');
        global $wpdb;
        
        
        set_time_limit(3000);
        $mysqli = new mysqli($host, $user, $pass, $name);
        $mysqli->select_db($name);
        $mysqli->query("SET NAMES 'utf8'");
        $queryTables = $mysqli->query('SHOW TABLES');
        while ($row = $queryTables->fetch_row()) {
            $target_tables[] = $row[0];
        } if ($tables !== false) {
            $target_tables = array_intersect($target_tables, $tables);
        }
        $content = "SET SQL_MODE = \"NO_AUTO_VALUE_ON_ZERO\";\r\nSET time_zone = \"+00:00\";\r\n\r\n\r\n/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;\r\n/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;\r\n/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;\r\n/*!40101 SET NAMES utf8 */;\r\n--\r\n-- Database: `" . $name . "`\r\n--\r\n\r\n\r\n";
        foreach ($target_tables as $table) {
            if (empty($table)) {
                continue;
            }
            $result = $mysqli->query('SELECT * FROM `' . $table . '`');
            $fields_amount = $result->field_count;
            $rows_num = $mysqli->affected_rows;
            $res = $mysqli->query('SHOW CREATE TABLE ' . $table);
            $TableMLine = $res->fetch_row();
            
            
            //$TableMLine[1] = str_ireplace($this->find_replace['options']['replace']['find_value'], $this->find_replace['options']['replace']['replace_value'], $TableMLine[1]);
            
            $content .= "\n\n" . $TableMLine[1] . ";\n\n";
            //$TableMLine[1] = str_ireplace('CREATE TABLE `', 'CREATE TABLE IF NOT EXISTS `', $TableMLine[1]);
            
            
            
            
            for ($i = 0, $st_counter = 0; $i < $fields_amount; $i++, $st_counter = 0) {
                while ($row = $result->fetch_row()) { //when started (and every after 100 command cycle):
                    if ($st_counter % 100 == 0 || $st_counter == 0) {
                        $content .= "\nINSERT INTO " . $table . " VALUES";
                    }
                    $content .= "\n(";
                    for ($j = 0; $j < $fields_amount; $j++) {
                        $row[$j] = str_replace("\n", "\\n", addslashes($row[$j]));
                        if (isset($row[$j])) {
                            $content .= '"' . $row[$j] . '"';
                        } else {
                            $content .= '""';
                        } if ($j < ($fields_amount - 1)) {
                            $content.= ',';
                        }
                    } $content .=")";
                    //every after 100 command cycle [or at last line] ....p.s. but should be inserted 1 cycle eariler
                    if ((($st_counter + 1) % 100 == 0 && $st_counter != 0) || $st_counter + 1 == $rows_num) {
                        $content .= ";";
                    } else {
                        $content .= ",";
                    } $st_counter = $st_counter + 1;
                }
            } $content .="\n\n\n";
        }
        $content .= "\r\n\r\n/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\r\n/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;\r\n/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;";
        
        ob_get_clean();
        $content = str_ireplace($this->find_replace['options']['replace']['find_value'], $this->find_replace['options']['replace']['replace_value'], $content);
        $content = str_replace($wpdb->prefix, 'xadbprefix_', $content);
        $file_path = $this->migration_zip_path . 'database.sql';
        $file = fopen($file_path, 'w');
        fwrite($file, $content);
        fclose($file);
        
    }

    
    
    public function generate_database_mysql_standard($host, $user, $pass, $name, $tables = false, $backup_name = false) {
        
        //require_once( ABSPATH . '/wp-load.php');
        global $wpdb;
        
        
        set_time_limit(3000);
        
        mysql_connect($host, $user, $pass) or die('Error connecting to MySQL server: ' . mysql_error());
        mysql_select_db($name) or die('Error selecting MySQL database: ' . mysql_error());
        mysql_query("SET NAMES 'utf8'");
        $queryTables = mysql_query('SHOW TABLES');
        while ($row = mysql_fetch_row($queryTables)) {
            $target_tables[] = $row[0];
        } if ($tables !== false) {
            $target_tables = array_intersect($target_tables, $tables);
        }
        $content = "SET SQL_MODE = \"NO_AUTO_VALUE_ON_ZERO\";\r\nSET time_zone = \"+00:00\";\r\n\r\n\r\n/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;\r\n/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;\r\n/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;\r\n/*!40101 SET NAMES utf8 */;\r\n--\r\n-- Database: `" . $name . "`\r\n--\r\n\r\n\r\n";
        foreach ($target_tables as $table) {
            if (empty($table)) {
                continue;
            }
            $result = mysql_query('SELECT * FROM `' . $table . '`');
            $fields_amount = mysql_num_fields($result);
            $rows_num = mysql_num_rows($result);
            $res = mysql_query('SHOW CREATE TABLE ' . $table);
            $TableMLine = mysql_fetch_row($res);
            
            
            //$TableMLine[1] = str_ireplace($this->find_replace['options']['replace']['find_value'], $this->find_replace['options']['replace']['replace_value'], $TableMLine[1]);
            
            $content .= "\n\n" . $TableMLine[1] . ";\n\n";
            //$TableMLine[1] = str_ireplace('CREATE TABLE `', 'CREATE TABLE IF NOT EXISTS `', $TableMLine[1]);
            
            
            
            
            for ($i = 0, $st_counter = 0; $i < $fields_amount; $i++, $st_counter = 0) {
                while ($row = mysql_fetch_row($result)) { //when started (and every after 100 command cycle):
                    if ($st_counter % 100 == 0 || $st_counter == 0) {
                        $content .= "\nINSERT INTO " . $table . " VALUES";
                    }
                    $content .= "\n(";
                    for ($j = 0; $j < $fields_amount; $j++) {
                        $row[$j] = str_replace("\n", "\\n", addslashes($row[$j]));
                        if (isset($row[$j])) {
                            $content .= '"' . $row[$j] . '"';
                        } else {
                            $content .= '""';
                        } if ($j < ($fields_amount - 1)) {
                            $content.= ',';
                        }
                    } $content .=")";
                    //every after 100 command cycle [or at last line] ....p.s. but should be inserted 1 cycle eariler
                    if ((($st_counter + 1) % 100 == 0 && $st_counter != 0) || $st_counter + 1 == $rows_num) {
                        $content .= ";";
                    } else {
                        $content .= ",";
                    } $st_counter = $st_counter + 1;
                }
            } $content .="\n\n\n";
        }
        $content .= "\r\n\r\n/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\r\n/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;\r\n/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;";
        
        ob_get_clean();
        $content = str_ireplace($this->find_replace['options']['replace']['find_value'], $this->find_replace['options']['replace']['replace_value'], $content);
        $content = str_replace($wpdb->prefix, 'xadbprefix_', $content);
        $file_path = $this->migration_zip_path . 'database.sql';
        $file = fopen($file_path, 'w');
        fwrite($file, $content);
        fclose($file);
        
    }
    
    
    // Function to remove folders and files 
    public function rrmdir($dir) {
        if (is_dir($dir)) {
            $files = scandir($dir);
            foreach ($files as $file)
                if ($file != "." && $file != "..")
                    $this->rrmdir("$dir/$file");
            rmdir($dir);
        }
        else if (file_exists($dir))
            unlink($dir);
    }

    public function Zippp($source, $destination) {
        if (!extension_loaded('zip') || !file_exists($source)) {
            return false;
        }

        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            return false;
        }

        $source = str_replace('\\', '/', realpath($source));

        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

            foreach ($files as $file) {
                $file = str_replace('\\', '/', $file);

                // Ignore "." and ".." folders
                if (in_array(substr($file, strrpos($file, '/') + 1), array('.', '..')))
                    continue;

                $file = realpath($file);

                if (is_dir($file) === true) {
                    $zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
                } else if (is_file($file) === true) {
                    $zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
                }
            }
        } else if (is_file($source) === true) {
            $zip->addFromString(basename($source), file_get_contents($source));
        }

        return $zip->close();
    }

}
