<?php

/**
 * Handles import.
 * 
 */
if (!defined('ABSPATH')) {
    exit;
}


if (!class_exists('XA_Migration_Batch_Importer', false)) {
    include_once( XA_MIGRATION_MAIN_PATH . '/importer/abstract-migration-batch-importer.php' );
}

class XA_Wp_Migration_Importer extends XA_Migration_Batch_Importer {

    protected $migration_zip_path;
    protected $migration_import_path;
    protected $warning_message;

    public function __construct() {
        parent::__construct();

        $this->migration_zip_path = plugin_dir_path(__FILE__) . '../../../xa-migration-export/'; //wp content path - fixed WP violation
        $this->migration_import_path = plugin_dir_path(__FILE__) . '../../../xa-migration-import/';
    }

    public function set_warning_message($msg = NULL) {
        update_option('xa_migration_import_warning_message', $msg);
    }

    public function get_warning_message() {
        return get_option('xa_migration_import_warning_message');
    }

    public function process_import() {
        $current_step = $this->get_page();
        switch ($current_step) {
            case 1:
                $this->upload_zip_file(); //upload zip file
                break;
            case 2:
                $this->handle_theme_extraction(); //extract themes zip
                break;
            case 3:
                $this->handle_uploads_extraction(); //extract uploads zip
                break;
            case 4:
                $this->handle_plugins_extraction(); //extract plugins zip
                break;
            case 5:
                //$this->xa_deactivate_plugins();
                //$this->handle_database_import(); //handle database import
                //$this->xa_flush_cache();
                break;
            default:
                break;
        }
    }

    protected function upload_zip_file() {

        if (!empty($_FILES['migration_file'])) {
            $filename = $_FILES['migration_file']['name'];
            $ext = pathinfo($filename, PATHINFO_EXTENSION);
            if( $ext !== 'zip'){
              $msg = 'Please upload a zip file';
              $this->set_warning_message($msg);  
            }else{
            $upload_details = $this->handle_upload();
            }
        } else {
            $msg = 'Please upload something substancial';
            $this->set_warning_message($msg);
        }

        if (!empty($upload_details) && true === $upload_details['status']) {
            $this->set_warning_message('');
            update_option('xa_migration_import_zip_details', $upload_details);
        } else {
            if(empty($msg))
            $this->set_warning_message('Upload failed - Please try again');
        }
    }

    protected function handle_theme_extraction() {

        $upload_details = get_option('xa_migration_import_zip_details');

        if (!empty($upload_details) && true === $upload_details['status']) {
            $file_path = $upload_details['details']['file'];
            require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
            $zip = new Zip();
            if (!file_exists($this->migration_import_path)) {
                $created = mkdir($this->migration_import_path, 0777, true);
                if (!$created) {
                    $warning_message = 'Permission denied - handle theme extraction';
                    $this->set_warning_message($warning_message);
                } else {
                    $this->set_warning_message('');
                }
            }
            $zip->unzip_file($file_path, $this->migration_import_path);
            if (file_exists($this->migration_import_path . 'xa-migration-export/themes.zip')) {
                $zip->unzip_file($this->migration_import_path . 'xa-migration-export/themes.zip', $this->migration_import_path . 'xa-migration-export/');
            } else {
                if (!file_exists($this->migration_import_path . 'xa-migration-export/themes/')) {
                    mkdir($this->migration_import_path . 'xa-migration-export/themes/', 0777, true);
                }
                $zip->unzip_file($this->migration_import_path . 'themes.zip', $this->migration_import_path . 'xa-migration-export/themes/');
            }
            $this->rcopy($this->migration_import_path . 'xa-migration-export/themes/', plugin_dir_path(__FILE__) . '../../../themes/');
        }
    }

    protected function handle_uploads_extraction() {


        require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
        $zip = new Zip();
        if (file_exists($this->migration_import_path . 'xa-migration-export/uploads.zip')) {
            $zip->unzip_file($this->migration_import_path . 'xa-migration-export/uploads.zip', $this->migration_import_path . 'xa-migration-export/');
        } else {
            if (!file_exists($this->migration_import_path . 'xa-migration-export/uploads/')) {
                mkdir($this->migration_import_path . 'xa-migration-export/uploads/', 0777, true);
            }
            $zip->unzip_file($this->migration_import_path . 'uploads.zip', $this->migration_import_path . 'xa-migration-export/uploads/');
        }
        $this->rcopy($this->migration_import_path . 'xa-migration-export/uploads/', plugin_dir_path(__FILE__) . '../../../uploads/');
    }

    protected function handle_plugins_extraction() {

        require_once XA_MIGRATION_MAIN_PATH . "src/zip.php";
        $zip = new Zip();

        if (file_exists($this->migration_import_path . 'xa-migration-export/plugins.zip')) {
            $zip->unzip_file($this->migration_import_path . 'xa-migration-export/plugins.zip', $this->migration_import_path . 'xa-migration-export/');
        } else {
            if (!file_exists($this->migration_import_path . 'xa-migration-export/plugins/')) {
                mkdir($this->migration_import_path . 'xa-migration-export/plugins/', 0777, true);
            }
            $zip->unzip_file($this->migration_import_path . 'plugins.zip', $this->migration_import_path . 'xa-migration-export/plugins/');
        }
        $this->rcopy($this->migration_import_path . 'xa-migration-export/plugins/', plugin_dir_path(__FILE__) . '../../../plugins/');
    }

    /*
    protected function handle_database_import() {
        $this->IMPORT_TABLES(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME, $this->migration_import_path . 'xa-migration-export/database.sql');
    }
     * 
     */

    public function handle_upload() {

        if (!function_exists('wp_handle_upload')) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }


        $uploadedfile = $_FILES['migration_file'];

        $upload_overrides = array('test_form' => false);

        $movefile = wp_handle_upload($uploadedfile, $upload_overrides);

        if ($movefile && !isset($movefile['error'])) {
            $msg = "File is valid, and was successfully uploaded.\n";
            $status = TRUE;
            return array('status' => $status, 'message' => $msg, 'details' => $movefile);
        } else {
            /**
             * Error generated by _wp_handle_upload()
             * @see _wp_handle_upload() in wp-admin/includes/file.php
             */
            $msg = $movefile['error'];
            $status = FALSE;
            return array('status' => $status, 'message' => $msg);
        }
    }

    // Function to remove folders and files 
    public function rrmdir($dir) {
        if (is_dir($dir)) {
            $files = scandir($dir);
            foreach ($files as $file)
                if ($file != "." && $file != "..")
                    $this->rrmdir("$dir/$file");
            rmdir($dir);
        }
        else if (file_exists($dir))
            unlink($dir);
    }

    // Function to Copy folders and files       
    public function rcopy($src, $dst) {
        /* if (file_exists($dst))
          $this->rrmdir($dst); */
        if (is_dir($src)) {
            if (!file_exists($dst))
                mkdir($dst, 0777, true);
            $files = scandir($src);
            foreach ($files as $file)
                if ($file != "." && $file != "..")
                    $this->rcopy("$src/$file", "$dst/$file");
        } else if (file_exists($src))
            copy($src, $dst);
    }

    /*
    public function IMPORT_TABLES($host, $user, $pass, $dbname, $sql_file) {

        set_time_limit(3000);
        $SQL_CONTENT = file_get_contents($sql_file);
        $allLines = explode("\n", $SQL_CONTENT);
        $mysqli = new mysqli($host, $user, $pass, $dbname);
        if (mysqli_connect_errno()) {
            echo "Failed to connect to MySQL: " . mysqli_connect_error();
        }
        $zzzzzz = $mysqli->query('SET foreign_key_checks = 0');
        preg_match_all("/\nCREATE TABLE(.*?)\`(.*?)\`/si", "\n" . $SQL_CONTENT, $target_tables);
        foreach ($target_tables[2] as $table) {
            $mysqli->query('DROP TABLE IF EXISTS ' . $table);
        } $zzzzzz = $mysqli->query('SET foreign_key_checks = 1');
        $mysqli->query("SET NAMES 'utf8'");
        $templine = ''; // Temporary variable, used to store current query
        foreach ($allLines as $line) {           // Loop through each line
            if (substr($line, 0, 2) != '--' && $line != '') {
                $templine .= $line;  // (if it is not a comment..) Add this line to the current segment
                if (substr(trim($line), -1, 1) == ';') {  // If it has a semicolon at the end, it's the end of the query
                    if (!$mysqli->query($templine)) {
                        print('Error performing query \'<strong>' . $templine . '\': ' . $mysqli->error . '<br /><br />');
                    } $templine = ''; // set variable to empty, to start picking up the lines after ";"
                }
            }
        } return 'Importing finished. Now, Delete the import file.';
    }
     * 
     */

    /**
     * Flush WP options cache
     *
     * @return void
     */
    function xa_flush_cache() {
        // Initialize WP cache
        wp_cache_init();

        // Flush WP cache
        wp_cache_flush();

        // Set WP cache
        wp_cache_set('alloptions', array(), 'options');
        wp_cache_set('notoptions', array(), 'options');

        // Delete WP cache
        wp_cache_delete('alloptions', 'options');
        wp_cache_delete('notoptions', 'options');
    }

    function xa_deactivate_plugins() {
        return update_option('active_plugins', array());
    }

}
