<?php

/**
  Plugin Name: WP Migration & Duplicator
  Plugin URI: https://wordpress.org/plugins/wp-migration-duplicator/
  Description: Migrate WordPress Contents and database quickly with ease.
  Author: markhf
  Version: 0.0.2
  Author URI:
  License: Under GPL2
  Text Domain: wordpress-migration-duplicator
 * 
 */
if (!defined('ABSPATH')) {
    exit;
}

if (!defined('XA_MIGRATION_JS_URL')) {
    define('XA_MIGRATION_JS_URL', plugin_dir_url(__FILE__) . 'assets/js/');
}
if (!defined('XA_MIGRATION_CSS_URL')) {
    define('XA_MIGRATION_CSS_URL', plugin_dir_url(__FILE__) . 'assets/css/');
}
if (!defined('XA_MIGRATION_IMG_URL')) {
    define('XA_MIGRATION_IMG_URL', plugin_dir_url(__FILE__) . 'assets/img/');
}
if (!defined('XA_MIGRATION_MAIN_URL')) {
    define('XA_MIGRATION_MAIN_URL', plugin_dir_url(__FILE__));
}

if (!defined('XA_MIGRATION_MAIN_PATH')) {
    define('XA_MIGRATION_MAIN_PATH', plugin_dir_path(__FILE__));
}

class Duplicator_Migrate_Base {

    const TEXT_DOMAIN = 'wordpress-migration-duplicator';
    const VERSION = '0.0.2';
    const PLUGN_BASE_PATH = __FILE__;

    public function __construct() {

        //add_action('init', array($this, 'include_related_files'));
        add_action('admin_menu', array($this, 'xa_register_migrate_submenu_page'));
        add_action('admin_menu', array($this, 'add_to_menus'));
        add_action('admin_head', array($this, 'hide_from_menus'));

        add_action('admin_init', array($this, 'download_export_file'));
        add_action('admin_init', array($this, 'completed_migration_process'));

        add_action('admin_enqueue_scripts', array($this, 'enqueue_export_scripts'));
        add_action('wp_ajax_xa_do_ajax_wp_export', array($this, 'xa_do_ajax_wp_export'));
        add_action('wp_ajax_xa_do_ajax_wp_import', array($this, 'xa_do_ajax_wp_import'));
        // Register xa exporters.
        $this->exporters['migration_exporter'] = array(
            'menu' => 'tools.php',
            'name' => __('Post Export', Duplicator_Migrate_Base::TEXT_DOMAIN),
            'capability' => 'edit_posts',
            'callback' => array($this, 'migration_exporter'),
        );
        $this->importers['migration_importer'] = array(
            'menu' => 'tools.php',
            'name' => __('Post Import', Duplicator_Migrate_Base::TEXT_DOMAIN),
            'capability' => 'edit_posts',
            'callback' => array($this, 'migration_importer'),
        );
        
        $this->migration_import_path = plugin_dir_path(__FILE__) . '../../xa-migration-import/';
    }

    public function xa_register_migrate_submenu_page() {
        add_submenu_page('tools.php', 'WP Migrator Export', 'WP Migrator Export', 'manage_options', 'xa-wordpress-export', array($this, 'xa_migrate_export_submenu_page_callback'));
        add_submenu_page('tools.php', 'WP Migrator Import', 'WP Migrator Import', 'manage_options', 'xa-wordpress-import', array($this, 'xa_migrate_import_submenu_page_callback'));
    }

    public function xa_migrate_export_submenu_page_callback() {
        require_once( 'views/exporter/html-admin-page-export.php');
    }

    public function xa_migrate_import_submenu_page_callback() {

        require_once( 'views/importer/html-admin-page-import.php');
    }

    public function include_related_files() {
        
    }

    public function enqueue_export_scripts() {

        $screen = get_current_screen();

        if ('tools_page_xa-wordpress-export' == $screen->id || 'tools_page_xa-wordpress-import' == $screen->id) {

            $my_js_version = date("ymd-Gis", filemtime(plugin_dir_path(__FILE__) . 'assets/js/xa-migrate-base.js'));
            wp_enqueue_script('xa_custom_js', XA_MIGRATION_JS_URL . 'xa-migrate-base.js', array(), $my_js_version);
            wp_localize_script('xa_custom_js', 'xa_export_params', array('export_nonce' => wp_create_nonce('xa-migration-export'),));
            wp_localize_script('xa_custom_js', 'xa_import_params', array('import_nonce' => wp_create_nonce('xa-migration-import'),));
            wp_enqueue_style('xa-wordpress-migrate', XA_MIGRATION_CSS_URL . 'xa-wordpress-migrate.css', array(), self::VERSION);
        }
    }

    // Batch exports via ajax

    public function xa_do_ajax_wp_export() {

        $find_replace_arr = array();
        parse_str($_POST['form'], $find_replace_arr);



        check_ajax_referer('xa-migration-export', 'security');

        if (!current_user_can('edit_posts')) {
            wp_die(-1);
        }

        include_once( XA_MIGRATION_MAIN_PATH . 'exporter/class-xa-migration-exporter.php' );

        $step = absint($_POST['step']);
        $exporter = new XA_Wp_Migration_Exporter ();
        $exporter->set_page($step);

        $exporter->find_replace = $find_replace_arr;

        $exporter->generate_zip_file();

        if (100 === $exporter->get_percent_complete()) {
            wp_send_json_success(array(
                'step' => 'done',
                'percentage' => 100,
                'url' => add_query_arg(array('nonce' => wp_create_nonce('xa-migration-export'), 'action' => 'download_migration_zip'), admin_url('tools.php?page=migration_exporter')),
            ));
        } else {
            wp_send_json_success(array(
                'step' => ++$step,
                'percentage' => $exporter->get_percent_complete(),
                'export_progress_message' => $exporter->get_progress_message(),
            ));
        }
    }

    public function final_export() {

        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            require_once "src/zip.php";
            $zip = new Zip();
            $zip->zip_start(plugin_dir_path(__FILE__) . '../../xa-migration-export.zip');
            $zip->zip_add(array(plugin_dir_path(__FILE__) . '../../xa-migration-export')); // can add multiple directories
            $zip_res = $zip->zip_end();
        } else {
            $this->Zippp(plugin_dir_path(__FILE__) . '../../xa-migration-export/', plugin_dir_path(__FILE__) . '../../xa-migration-export.zip');
        }
    }

// add menu items for exporter.

    public function add_to_menus() {

        foreach ($this->exporters as $id => $exporter) {
            add_submenu_page($exporter['menu'], $exporter['name'], $exporter['name'], $exporter['capability'], $id, $exporter['callback']);
        }
        foreach ($this->importers as $id => $exporter) {
            add_submenu_page($exporter['menu'], $exporter['name'], $exporter['name'], $exporter['capability'], $id, $exporter['callback']);
        }
    }

    // hide menu items from view - ajax page.

    public function hide_from_menus() {
        global $submenu;

        foreach ($this->exporters as $id => $exporter) {
            if (isset($submenu[$exporter['menu']])) {
                foreach ($submenu[$exporter['menu']] as $key => $menu) {
                    if ($id === $menu[2]) {
                        unset($submenu[$exporter['menu']][$key]);
                    }
                }
            }
        }

        foreach ($this->importers as $id => $importer) {
            if (isset($submenu[$importer['menu']])) {
                foreach ($submenu[$importer['menu']] as $key => $menu) {
                    if ($id === $menu[2]) {
                        unset($submenu[$importer['menu']][$key]);
                    }
                }
            }
        }
    }

    // export page html.

    public function migration_exporter() {
        //include_once( XA_MIGRATION_MAIN_PATH . '/exporter/class-xa-migration-exporter.php' );
        error_reporting(E_ALL ^ E_WARNING); // wp-includes/pluggable line 914-915 throwing warning - need to backtrace where it is coming from
        //ini_set("display_errors", 1);
        require_once( 'views/exporter/html-download-export-page.php');
    }

    public function download_export_file() {

        if (isset($_GET['action'], $_GET['nonce']) && wp_verify_nonce($_GET['nonce'], 'xa-migration-export') && 'download_migration_zip' === $_GET['action']) {

            $this->final_export();
        }
    }

    // import page html.

    public function migration_importer() {
        //$this->handle_database_import();
        $user = get_current_user_id();
        if (file_exists($this->migration_import_path . 'xa-migration-export/database.sql')) {
            $sql_file_path = $this->migration_import_path . 'xa-migration-export/database.sql';
        } else {
            $sql_file_path = $this->migration_import_path . 'database.sql';
        }
        global $wpdb;

        $process_fail_message = $process_status = '';
        // check database client
        if (empty($wpdb->use_mysqli)) {
            if(file_exists($sql_file_path)){
            $process_status = $this->IMPORT_MYSQL_TABLES(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME, $sql_file_path, $wpdb->preffix);
            }else{
              $process_fail_message =  'Database file does not exist - please try again';
            }
        } else {
            if(file_exists($sql_file_path)){
            $process_status = $this->IMPORT_MYSQLI_TABLES(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME, $sql_file_path, $wpdb->prefix);
            }else{
               $process_fail_message =  'Database file does not exist - please try again'; 
            }
        }



        if (( $user)) {
            wp_set_current_user($user);
            wp_set_auth_cookie($user);
        }

        require_once( 'views/importer/html-import-complete-page.php');
    }

    public function completed_migration_process() {

        if (isset($_GET['action'], $_GET['nonce']) && 'completed_migration_process' === $_GET['action']) {

            //$this->final_export(); do the final process after import
        }
    }

    public function IMPORT_MYSQLI_TABLES($host, $user, $pass, $dbname, $sql_file, $current_prefix) {


        set_time_limit(3000);
        $SQL_CONTENT = file_get_contents($sql_file);
        $allLines = explode("\n", $SQL_CONTENT);
        $mysqli = new mysqli($host, $user, $pass, $dbname);
        if (mysqli_connect_errno()) {
            echo "Failed to connect to MySQL: " . mysqli_connect_error();
        }
        //$mysqli->begin_transaction();
        /* set autocommit to off */
        $mysqli->autocommit(FALSE);

        $zzzzzz = $mysqli->query('SET foreign_key_checks = 0');
        if (!$zzzzzz) {
            $mysqli->rollback();
            return false;
        }
        preg_match_all("/\nCREATE TABLE(.*?)\`(.*?)\`/si", "\n" . $SQL_CONTENT, $target_tables);


        if ($result = $mysqli->query("SHOW TABLES")) {
            while ($row = $result->fetch_array(MYSQLI_NUM)) {
                $drop_tbl = $mysqli->query('DROP TABLE IF EXISTS ' . $row[0]);
                if (!$drop_tbl) {
                    $mysqli->rollback();
                    return false;
                }
            }
        }

        /*
          foreach ($target_tables[2] as $table) {
          $transaction = $mysqli->query('DROP TABLE IF EXISTS ' . $table);
          if (!$transaction) {
          $mysqli->rollback();
          return false;
          }
          }
         * 
         */
        $zzzzzz = $mysqli->query('SET foreign_key_checks = 1');
        if (!$zzzzzz) {
            $mysqli->rollback();
            return false;
        }
        $mysqli->query("SET NAMES 'utf8'");
        $templine = ''; // Temporary variable, used to store current query
        foreach ($allLines as $line) {           // Loop through each line
            if (substr($line, 0, 2) != '--' && $line != '') {
                $templine .= $line;  // (if it is not a comment..) Add this line to the current segment
                if (substr(trim($line), -1, 1) == ';') {  // If it has a semicolon at the end, it's the end of the query
                    $templine = str_replace('xadbprefix_', $current_prefix, $templine);
                    if (!$mysqli->query($templine)) {
                        print('Error performing query \'<strong>' . $templine . '\': ' . $mysqli->error . '<br /><br />');
                        $mysqli->rollback();
                        return false;
                    } $templine = ''; // set variable to empty, to start picking up the lines after ";"
                }
            }
        }
        $mysqli->commit();
        //return 'Importing finished. Now, Delete the import file.';
        return true;
    }

    public function IMPORT_MYSQL_TABLES($host, $user, $pass, $dbname, $sql_file, $current_prefix) {


        set_time_limit(3000);
        $SQL_CONTENT = file_get_contents($sql_file);
        $allLines = explode("\n", $SQL_CONTENT);

        mysql_connect($host, $user, $pass) or die('Error connecting to MySQL server: ' . mysql_error());

        mysql_select_db($dbname) or die('Error selecting MySQL database: ' . mysql_error());

        mysql_query("BEGIN");

        $zzzzzz = mysql_query('SET foreign_key_checks = 0');
        if (!$zzzzzz) {
            mysql_query("ROLLBACK");
            return false;
        }
        preg_match_all("/\nCREATE TABLE(.*?)\`(.*?)\`/si", "\n" . $SQL_CONTENT, $target_tables);

        if ($result = mysql_query("SHOW TABLES")) {
            while ($row = mysql_fetch_array($result, MYSQL_NUM)) {
                $drop_tbl = mysql_query('DROP TABLE IF EXISTS ' . $row[0]);
                if (!$drop_tbl) {
                    mysql_query("ROLLBACK");
                    return false;
                }
            }
        }

        /*
          foreach ($target_tables[2] as $table) {
          $transaction = mysql_query('DROP TABLE IF EXISTS ' . $table);
          if (!$transaction) {
          mysql_query("ROLLBACK");
          return false;
          }
          }
         * 
         */
        $zzzzzz = mysql_query('SET foreign_key_checks = 1');
        if (!$zzzzzz) {
            mysql_query("ROLLBACK");
            return false;
        }
        $mysqli->query("SET NAMES 'utf8'");
        $templine = ''; // Temporary variable, used to store current query
        foreach ($allLines as $line) {           // Loop through each line
            if (substr($line, 0, 2) != '--' && $line != '') {
                $templine .= $line;  // (if it is not a comment..) Add this line to the current segment
                if (substr(trim($line), -1, 1) == ';') {  // If it has a semicolon at the end, it's the end of the query
                    $templine = str_replace('xadbprefix_', $current_prefix, $templine);
                    if (!mysql_query($templine)) {
                        print('Error performing query \'<strong>' . $templine . '\': ' . mysql_error() . '<br /><br />');
                        mysql_query("ROLLBACK");
                        return false;
                    } $templine = ''; // set variable to empty, to start picking up the lines after ";"
                }
            }
        }
        mysql_query("COMMIT");
        //return 'Importing finished. Now, Delete the import file.';
        return true;
    }

    // Batch import via ajax

    public function xa_do_ajax_wp_import() {


        check_ajax_referer('xa-migration-import', 'security');

        if (!current_user_can('edit_posts')) {
            wp_die(-1);
        }

        include_once( XA_MIGRATION_MAIN_PATH . 'importer/class-xa-migration-importer.php' );

        $step = absint($_POST['step']);

        $importer = new XA_Wp_Migration_Importer();
        $importer->set_page($step);

        $importer->process_import();

        if (100 === $importer->get_percent_complete()) {

            
            wp_send_json_success(array(
                'step' => 'done',
                'message' => $importer->get_warning_message(),
                'percentage' => 100,
                'url' => add_query_arg(array('action' => 'completed_migration_process'), admin_url('tools.php?page=migration_importer')),
            ));
        } else {
            wp_send_json_success(array(
                'step' => ++$step,
                'message' => $importer->get_warning_message(),
                'percentage' => $importer->get_percent_complete(),
                'export_progress_message' => $importer->get_progress_message(),
            ));
        }
    }

    public function Zippp($source, $destination) {
        if (!extension_loaded('zip') || !file_exists($source)) {
            return false;
        }

        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            return false;
        }

        $source = str_replace('\\', '/', realpath($source));

        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

            foreach ($files as $file) {
                $file = str_replace('\\', '/', $file);

                // Ignore "." and ".." folders
                if (in_array(substr($file, strrpos($file, '/') + 1), array('.', '..')))
                    continue;

                $file = realpath($file);

                if (is_dir($file) === true) {
                    $zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
                } else if (is_file($file) === true) {
                    $zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
                }
            }
        } else if (is_file($source) === true) {
            $zip->addFromString(basename($source), file_get_contents($source));
        }

        return $zip->close();
    }

}

new Duplicator_Migrate_Base();
